(**********************************************************)
(*                      VMEM.PAS                          *)
(*                                                        *)
(*  Author   : M.T.Shiang                                 *)
(*                                                        *)
(*  Date     : 1994/11/21                                 *)
(*                                                        *)
(*  File name: VMEM.PAS                                   *)
(*                                                        *)
(*  Purpose  : Unit for the management of "VHEAP".        *)
(*                                                        *)
(*  Reference: VDEVICE                                    *)
(**********************************************************)

Unit VMEM;

Interface

(*
Uses VDEVICE;  { This unit employes some routines in unit "VDEVICE"}
*)

Const
  VMEMBlocks = 1024;    { The maximum size of one heap is 16M bytes }
                        { 1024 blocks * 16kb/block = 16M }
  VMEMHandles = 1024;   { Maximum number of handles }

Type
  { Status of VHeap }
  VMEMStatus = (Ok, NotInit, NotEnoughMem, OutOfMem,
                VMEM_InvalidInit);

  { Type of device that VDEVICE uses }
  DeviceType = (Exp,         { Expanded memory }
                Dsk);        { Disk file }

  { Definition of virtual-heap }
  VHeapPtr = ^VHeap;                             { Pointer to VHeap }
  VHeap = Record
    ID: Word;                                    { Identifier code }
    Blocks: Array[1..VMEMBlocks] Of Word;        { VHeap control array }
    BlockCount: Integer;                         { Number of blocks }
  End;

{ Global variables  }
Const
  VMEMInit: Boolean = False; { VMEMInit = True, if system initialized }
                             { VMEMInit = False, if system not initialized }

Var
  PoolSize: Word;            { Size of memory pool in blocks }
  FileName: String;          { File name of disk file }
  FileHanle: Word;           { Handle of disk file }
  FileSize: Word;            { Size of disk memory in blocks }

Implementation

(************************************************************************)
(* Function: Install VEMM system                                        *)
(*                                                                      *)
(* Input: Size:Word;    Size of memory pool in blocks                   *)
(*                                                                      *)
(* Return: The status of VMEM system (success, or error code)           *)
(*                                                                      *)
(* External: Name                   Source                              *)
(*           Writeln                Run-Time Library                    *)
(************************************************************************)
Function InitVMEM(Size: Word): VMemStatus;
Begin
  { If VMEM system is already initialized or the size of memory pool }
  { equals zero, then return VMEM_InvalidInit }
  If (VMEMInit) Or (Size = 0) Then
    Begin
      InitVMEM := VMEM_InvalidInit;
      Exit;
    End;

  BlockUsed := 0;




(************************************************************************)
(* Function: Transfer data from dos heap to Virtual-Heap                *)
(*                                                                      *)
(* Input:                                                               *)
(*         Var Heap: VHeap;  Virtual-Heap                               *)
(*         Var Data;         Address of data that will be transfered    *)
(*         Count: Longint;   Number of bytes to transfer                *)
(*         Dest: Longint;    Destination of virtual-memory              *)
(*                                                                      *)
(* Output:                                                              *)
(*         Status of VMEM                                               *)
(*                                                                      *)
(* External:                                                            *)
(*                                                                      *)
(************************************************************************)
Function WriteVHeap(Var Heap: VHeap;
                    Var Data;
                    Count: Longint;
                    Dest: Longint): VMemStatus;
Var
  Block: Word;
  Offset: Word;
Begin
  { Convert linear address into the form of Block:Offset }
  Block := Dest SHR 14;
  Offset := Dest - Block SHL 14;

  { If "Block" is great than the total number of blocks in "Heap" }
  { ,we must
  If (Block > Heap.BlockCount) Then
    Begin
      WriteHeap := VMEM_Invalid;
      Exit;
    End;



{*----------------------------------------------------------------------------*{
{* \ G w POWRSTOR t                                                 *{
{*                                                                            *{
{* Ѽ G unsigned          NeededSize :  memory pool jp (block Ӽ)    *{
{*         union INITelement *initArray :  ]w memory pool һݪul}Cv*{
{*         int               infoflag   :  O_nXT.                  *{
{*                                         0  ܤn                        *{
{*                                         1  ܭn                          *{
{*                                                                            *{
{* j G POWRSTOR A (\Aο~NX)                                 *{
{*                                                                            *{
{* ~` G  W                    ӷ                                  *{
{*              printf()                C  Run Time {w                    *{
{*              getch()                 C  Run Time {w                    *{
{*              sprintf()               C  Run Time {w                    *{
{*                                                                            *{
{*              EXPsize()               POWRDEVC    {w                    *{
{*              EXPframe()              POWRDEVC    {w                    *{
{*              EXTsize()               POWRDEVC    {w                    *{
{*              filSize()               POWRDEVC    {w                    *{
{*              shellinfo2file()        POWRDEVC    {w                    *{
{*                                                                            *{
{*              QuitPwr()               POWRSTOR    {w                    *{
{*              Blocks_status_display() POWRSTOR    {w                    *{
{*----------------------------------------------------------------------------*{
STORstatus InitPwr( unsigned          NeededSize,
                    union INITelement *initArray,
                    int               infoflag
                  )
{
unsigned    i,j, ib;         {* Blocks[] }C (index) *{
char        msg[80];         {* ǵTɪT            *{
unsigned    DiskNeededSize;  {* ϺаO骺ݨDq          *{
STORstatus  stat;            {* POWRSTOR A               *{


    {*  pG POWRSTOR wgwˡAǦ^ InvalInit NX *{
    if( pwrInitialized ) return InvalInit;

    {*  pGnD memory pool jpOsAǦ^ InvalInit NX *{
    if( NeededSize == 0 ) return InvalInit;

    {*  pGnD memory pool j POWRSTOR OAǦ^ InvalInit NX *{
    if( NeededSize > VMEMBLOCKS ) return InvalInit;


    {* ˬd]w memory poolɩһݪul}CvAeO_OXk]w *{
    i = 0;
    while( i < 16 )   {* ul}CvjpO 16 *{
        switch( initArray[i].Nulmem.memType )    {* O]wA *{
        {
            {* pGOAXiAκϺаOAh i Ȳ֥[ 1 *{
            case ExpMem  :
            case ExtMem  :
            case DskFile :
                           i += 1;
                           break;

            case NulMem  : {* аO *{
                           if ( i > 0 ) i = 17;
                           else         i = 18;
                           break;

            default      : i = 65535;
                           break;
        }

    if( i == 16 )      {* ul}CvӦh *{
       return BadInitData;

    if( i == 65535 )   {* ul}CvXkO髬A]w *{
       return BadInitData;

    if( i == 18 )      {* ul}CvӤ *{
       return InvalInit;

    stat = Okay;       {* l}CeXWw *{



    #if (STORDEBUG)         {* q, CLXUƤe *{
    Blocks_status_display();
    #endif



    i = 0;
    ib = 0;                  {* uPOWRSTOR ޲z}Cv (index) ]wb_Y *{
    blksUsed = 0;            {* uPOWRSTOR ޲z}CvQϥΪ block Ӽ     *{
    POOLsize = NeededSize ;  {* memory pool ڤjp^HݨDUOjp *{

    while( (initArray[i].Nulmem.memType != NulMem) &&  {*    wC@ӡul}Cv *{
           (NeededSize > 0) &&                         {* B memory pool js         *{
           (stat == Okay)      )                       {* B Xkul}Cv         *{
    {
        switch( initArray[i].Nulmem.memType )  {  {* w藍PO髬A *{

            case ExpMem  : {* XiO *{

                 {* tmXiO *{
                 EXPsize(initArray[i].EXPmem.Rsrvd, NeededSize,\
                         &ExpHandle, &ExpSize);

                 if (ExpSize == 0)       {* StmXiO *{
                     break;

                 ExpFrame = EXPframe();  {* DXiO骺ئ} *{

                 ExpFlag = ib;  {* ExpFlag ]wVuPOWRSTOR ޲z}CvXiO骺_lB *{


                 #if (STORDEBUG)  {* q, CLXUƤe *{
                 printf("initArray[%d]: ExpMem. Rsrvd = %u  NeededSize = %u \n", \
                         i, initArray[i].EXPmem.Rsrvd, NeededSize );
                 printf("               ExpHandle = %u  ExpFrame = \%x   ExpSize = %u  ExpFlag = %u \n",
                         ExpHandle, ExpFrame, ExpSize, ExpFlag );
                 #endif


                 for(j=ib; j< ib+ExpSize ; j++)  {* NoXiOOuPOWRSTOR ޲z}Cv *{
                     Blocks[j].type = EXP_TYPE;

                 ib += ExpSize;          {* ץuPOWRSTOR ޲z}Cv   *{
                 NeededSize -= ExpSize;  {* ץ|ݰtmUO block  *{

                 #if (STORDEBUG)  {* q, CLXUƤe *{
                 printf("NeededSize = %u \n", NeededSize );
                 getch() ;
                 #endif

                 break;

            case ExtMem  :  {* O *{

                 ExtSize = EXTsize();  {* O骺jpA쬰 K *{

                 {* p⩵O򩳦} (줣O KAЪ`N) *{
                 ExtBase = (initArray[i].EXTmem.LoLimit << 6);

                 ExtFlag = ib;  {* ExpFlag ]wVuPOWRSTOR ޲z}CvO骺_lB *{


                 #if (STORDEBUG)  {* q, CLXUƤe *{
                 printf("initArray[%d]: NeededSize = %u  ExtLoLimit = %u  ExtHiLimit = %u \n", \
                         i, NeededSize, initArray[i].EXTmem.LoLimit, initArray[i].EXTmem.HiLimit );
                 printf("               ExtSize = %u  ExtFlag = %u  ExtBase = %u  \n",
                         ExtSize, ExtFlag, ExtBase );
                 #endif


                 ExtSize >>= 4 ;  {* NO骺jp K ഫ blocks *{

                 if (initArray[i].EXTmem.LoLimit >= 64) {
                     if (initArray[i].EXTmem.HiLimit == 0) {
                         {* ܱNBΩҦO *{
                         ExtSize = (ExtSize >= NeededSize) ? NeededSize : ExtSize ;
                     }
                     else if ((initArray[i].EXTmem.HiLimit - initArray[i].EXTmem.LoLimit + 1) >= 4)  {
                         {* ܶȹBΤ@O *{
                         ExtSize = (initArray[i].EXTmem.HiLimit - initArray[i].EXTmem.LoLimit + 1);
                         ExtSize = (ExtSize >= NeededSize) ? NeededSize : ExtSize ;
                     }
                     else {  {* BΩO memory pool  *{
                         ExtSize = 0;
                     }
                 }
                 else { {* EXTmem.LoLimit < 64. YBΩO *{
                     ExtSize = 0;
                 }


                 for(j=ib; j< ib+ExtSize; j++)  {* NoǩOOuPOWRSTOR ޲z}Cv *{
                     Blocks[j].type = EXT_TYPE;

                 ib += ExtSize;          {* ץuPOWRSTOR ޲z}Cv   *{
                 NeededSize -= ExtSize;  {* ץ|ݰtmUO block  *{

                 #if (STORDEBUG) {* q, CLXUƤe *{
                 printf("NeededSize = %u \n", NeededSize );
                 getch() ;
                 #endif

                 break;

            case DskFile :  {* ϺаO *{

                 {* pӯDhֺϺЪŶ *{
                 {* YݨDqj]w̤j\qANγ̤j\q *{
                 if ( NeededSize > initArray[i].diskF.MaxSize )
                      DiskNeededSize = initArray[i].diskF.MaxSize;
                 else
                      DiskNeededSize = NeededSize;

                 {* tmϺЪŶ *{
                 FileSize = filSize(initArray[i].diskF.namestring, \
                                    &FileHandle, DiskNeededSize) ;

                 FileFlag = ib;  {* ExpFlag ]wVuPOWRSTOR ޲z}CvϺаO骺_lB *{


                 #if (STORDEBUG)  {* q, CLXUƤe *{
                 printf("initArray[%d]: NeededSize = %u  MaxSize = %u  namestring = %s \n", \
                         i, NeededSize, initArray[i].diskF.MaxSize, initArray[i].diskF.namestring );
                 printf("               FileSize = %u  DiskNeededSize = %u \n", FileSize, DiskNeededSize );
                 #endif



                 for(j=ib; j< ib+FileSize; j++)  {* NoǺϺаOOuPOWRSTOR ޲z}Cv *{
                     Blocks[j].type = DSK_TYPE;


                 ib += FileSize;          {* ץuPOWRSTOR ޲z}Cv   *{
                 NeededSize -= FileSize;  {* ץ|ݰtmUO block  *{

                 #if (STORDEBUG)  {* q, CLXUƤe *{
                 printf("NeededSize = %u \n", NeededSize );
                 getch() ;
                 #endif

                 break;
        }   {* switch() *{

        i++ ;   {* ץul}CvСAHKiU@Ӱtmʧ@ *{

        if (NeededSize == 0)    {* OwtmAݦAiUh *{
        break;
    }


    stat = Okay;  {* ]wAAܻUOw˸mtm\ *{


    if( (stat != Okay) ||         {*   w memory pool \A        *{
        (NeededSize > 0) )        {* λUO餣ݨDq       *{
    {
        QuitPwr();                {* } POWRSTOR tΡAҦO *{
        if( stat == Okay )
           stat = NotEnoughMem;   {* ]w POWRSTOR ANX           *{
        return stat;              {* Ǧ^AȵIs               *{
    }


    {* XTɮ *{
    if (infoflag == 1) {
        sprintf(msg,"POWRSTOR installed !!\n");
        shellinfo2file(msg);    {* ǰeTܸT *{
        sprintf(msg,"POWRSTOR use total     %4u Blocks, %8u K \n",POOLsize,POOLsize*16);
        shellinfo2file(msg);    {* ǰeTܸT *{
        sprintf(msg,"         Ext used :   %4u Blocks, %8u K \n",ExtSize,ExtSize*16);
        shellinfo2file(msg);    {* ǰeTܸT *{
        if (ExtSize != 0)  {
            sprintf(msg,"             Base :   %4u  \n",ExtBase);
            shellinfo2file(msg);    {* ǰeTܸT *{
        }
        sprintf(msg,"         Exp used :   %4u Blocks, %8u K \n",ExpSize,ExpSize*16);
        shellinfo2file(msg);    {* ǰeTܸT *{
        sprintf(msg,"         Dsk used :   %4u Blocks, %8u K \n",FileSize,FileSize*16);
        shellinfo2file(msg);    {* ǰeTܸT *{
    }




    #if (STORDEBUG)   {* q, CLXUƤe *{
    Blocks_status_display();
    #endif

    pwrInitialized = true;  {* ]wuw˺XСvA POWRSTOR ww˧ *{

    return Okay;            {* \A^Ȭ Okay *{

}
{*----------------------------------------------------------------------------*{
{* \ G  } POWRSTOR t                                                *{
{*          ҦgQtmUO駡NbƤ                      *{
{*                                                                            *{
{* Ѽ G  L                                                                *{
{*                                                                            *{
{* j G L                                                               *{
{*                                                                            *{
{* ~` G  W                    ӷ                                  *{
{*              EXTquit()               POWRDEVC  {w                      *{
{*              EXPquit()               POWRDEVC  {w                      *{
{*              filQuit()               POWRDEVC  {w                      *{
{*----------------------------------------------------------------------------*{
void QuitPwr(void)
{
    unsigned i;

    if( pwrInitialized )            {* POWRSTOR wgw *{
    {
        if (ExpFlag != 65535)       {* memory pool XiO *{
            EXPquit( ExpHandle ) ;  {* XiO             *{

        if (ExtFlag != 65535)       {* memory pool O *{
            EXTquit( ) ;            {* 񩵦O             *{

        if (FileFlag != 65535)      {* memory pool ϺаO *{
            filQuit( FileHandle ) ; {* ϺаO             *{

        pwrInitialized = false;     {* ]ww˺XЬs           *{
    }

}
{*----------------------------------------------------------------------------*{
{* \ G  memory pool wQϥΪŶjp                              *{
{*                                                                            *{
{* Ѽ G unsigned long  *bytesUsed  :  ѼƱNǦ^wϥΤŶ (bytes)     *{
{*                                                                            *{
{* j G    POWRSTOR A                                                 *{
{*                                                                            *{
{* ~` G  W                    ӷ                                  *{
{*              L                                                            *{
{*                                                                            *{
{* `N G  ѩ POWRSTOR OH block AҥH *bytesUsed NO           *{
{*          16384 (16K) ƭ                                              *{
{*----------------------------------------------------------------------------*{
STORstatus STORused( unsigned long  *bytesUsed )
{
    if( pwrInitialized == false )    {* POWRSTOR |w *{
        return NotInitialized;

    *bytesUsed = blksUsed;  {* memory pool QϥΪ block Ӽ *{

    *bytesUsed <<= 14;      {*  block ഫ bytes *{

    return Okay;
}
{*----------------------------------------------------------------------------*{
{* \ G   memory pool ѾliΪŶ                                 *{
{*                                                                            *{
{* Ѽ G  unsigned long  *bytesAvailable  : ѼƱNǦ^iΪŶ (bytes)    *{
{*                                                                            *{
{* j G    POWRSTOR A                                                 *{
{*                                                                            *{
{* ~` G  W                    ӷ                                  *{
{*              L                                                            *{
{*                                                                            *{
{* `N G  ѩ POWRSTOR OH block AҥH *bytesAvailable NO      *{
{*          16384 (16K) ƭ                                              *{
{*----------------------------------------------------------------------------*{
STORstatus STORavai( unsigned long  *bytesAvailable )
{
    if( pwrInitialized == false )   {* POWRSTOR |w *{
        return NotInitialized;

    *bytesAvailable = POOLsize-blksUsed;  {* memory pool Ѿli block Ӽ *{

    *bytesAvailable <<= 14;    {*  block ഫ bytes *{

    return Okay;
}
{*----------------------------------------------------------------------------*{
{* \ G  ͤ@ STORheapAjps                                       *{
{*                                                                            *{
{* Ѽ G  struct STORheap  *heapID : ѼƫV STORheap ƵcA       *{
{*                                     NǦ^ STORheap ҦTC           *{
{*                                                                            *{
{* j G    POWRSTOR A                                                 *{
{*                                                                            *{
{* ~` G  W                    ӷ                                  *{
{*              L                                                            *{
{*----------------------------------------------------------------------------*{
STORstatus newSheap( struct STORheap  *heapID )
{
int i;

    if( pwrInitialized == false ) {* POWRSTOR |w *{
        return NotInitialized;


    {* NܼƲMb *{
    heapID->actblocks = 0;
    for (i=0 ; i< STORheapBLOCKS; i++)
         heapID->blocks[i] = 0;


    {* tm STORheap ѧOX *{
    heapID->ID = curID;


    curID ++ ;   {* ֭pѧOX *{


    return Okay;
}
{*----------------------------------------------------------------------------*{
{* \ G   ]w STORheap Ŷjp                                         *{
{*                                                                            *{
{* Ѽ G   struct STORheap  *heapID :  STORheap Ƶc                  *{
{*           unsigned long    newSize :  STORheap Ŷjp (bytes)          *{
{*                                                                            *{
{* j G    POWRSTOR A                                                 *{
{*                                                                            *{
{* ~` G  W                                  ӷ                    *{
{*              Blocks_status_display();              POWRSTOR  {w        *{
{*              heapID_status_display(*heapID);       POWRSTOR  {w        *{
{*----------------------------------------------------------------------------*{
STORstatus  setSheap( struct STORheap  *heapID,
                      unsigned long    newSize
                    )
{
int        counter = 0;
unsigned   i;
int        enoughflag = false;


    if( pwrInitialized == false ) {* POWRSTOR |w *{
        return NotInitialized;


    {*  bytes ഫ blocks *{
    heapID->actblocks = (newSize >> 14) + 1;   {* i|O@ blocks *{


    {* memory pool O_Ŷ ? *{
    for(i=0, counter=0; i< VMEMBLOCKS; i++) {
        if ( Blocks[i].flag.used == 0 )   {   {*  block |Qϥ   *{
             counter ++ ;                     {* iΪ block Ӽƥ[ 1 *{
        }

        if (counter == heapID->actblocks) {   {* Ŷ *{
            enoughflag = true;
            break;
        }
    }


    if (! enoughflag)   {* memory pool Ŷ *{
        return NotEnoughMem;

    {* tm memory pool OŶ STORheap  *{
    for(i=0,counter=0; i< VMEMBLOCKS; i++) {
        if ( Blocks[i].flag.used == 0 )   {    {*  block |Qϥ        *{
             heapID->blocks[ counter ] = i;    {* O STORheap ޲z}C *{
             counter ++ ;                      {* ֭pwtm block Ӽ    *{
             Blocks[i].flag.ID = heapID->ID ;  {* OѧOX                 *{
             Blocks[i].flag.used = 1 ;         {* ܨϥΪA               *{
        }

        if (counter == heapID->actblocks)      {* Ŷ *{
            break;
    }



    {*  memory pool jp *{
    blksUsed  +=  heapID->actblocks ;


    #if (STORDEBUG)  {* q, CLXUƤe *{
    Blocks_status_display();
    heapID_status_display(*heapID);
    #endif


    return Okay;
}
{*----------------------------------------------------------------------------*{
{* \ G R@ STORheapAO鵹 memory pool                      *{
{*                                                                            *{
{* Ѽ G struct STORheap  *heapID : STORheap Ƶc                     *{
{*                                                                            *{
{* j G    POWRSTOR A                                                 *{
{*                                                                            *{
{* ~` G  W                    ӷ                                  *{
{*              printf()                C  Run Time {w                    *{
{*              Blocks_status_display() POWRSTOR    {w                    *{
{*              heapID_status_display() POWRSTOR    {w                    *{
{*----------------------------------------------------------------------------*{
STORstatus  delSheap( struct STORheap  *heapID )
{
unsigned int  i;
unsigned int  index;


    if( pwrInitialized == false ) {* POWRSTOR |w *{
        return NotInitialized;


    #if (STORDEBUG)  {* q, CLXUƤe *{
    printf("delSheap(). \n");
    heapID_status_display(*heapID);
    #endif




    for(i=0; i< heapID->actblocks; i++)  {  {* w STORheap ޲z}CC@Ӥ *{
        index = heapID->blocks[i];
        Blocks[ index ].flag.used = 0;      {* ܨϥΪA  *{
    }



    #if (STORDEBUG)  {* q, CLXUƤe *{
    Blocks_status_display();
    #endif


    {*  memory pool jp *{
    blksUsed -= heapID->actblocks;

    {*  STORheap jp *{
    heapID->actblocks = 0;

    return Okay;
}
{*----------------------------------------------------------------------------*{
{* \ G W[ STORheap Ŷjp                                           *{
{*                                                                            *{
{*         줺evT                                                     *{
{*                                                                            *{
{* Ѽ G struct STORheap  *heapID     : STORheap Ƶc                 *{
{*         unsigned long    size        : W[Ŷjp (bytes)            *{
{*         unsigned long    *newSegBase :                                     *{
{*                                                                            *{
{* j G    POWRSTOR A                                                 *{
{*                                                                            *{
{* ~` G  W                    ӷ                                  *{
{*              printf()                C  Run Time {w                    *{
{*              Blocks_status_display() POWRSTOR    {w                    *{
{*              heapID_status_display() POWRSTOR    {w                    *{
{*----------------------------------------------------------------------------*{
STORstatus  aloSheap( struct STORheap  *heapID,
                      unsigned long    size,
                      unsigned long    *newSegBase
                    )
{
int  counter = 0;
unsigned incblocks;
unsigned   i;
int  enoughflag = false;



    {*  bytes ഫ blocks *{
    incblocks = (size >> 14) + 1;   {* i|O@ blocks *{


    {* memory pool O_Ŷ ? *{
    for(i=0, counter=0; i< VMEMBLOCKS; i++) {
        if ( Blocks[i].flag.used == 0 )   {     {*  block |Qϥ *{
             counter ++ ;                       {* iΪ block Ӽƥ[ 1 *{
        }

        if (counter == incblocks) {             {* Ŷ *{
            enoughflag = true;
            break;
        }
    }


    if (! enoughflag)             {* memory pool Ŷ *{
        return NoMoreMem;


    {* tm memory pool OŶ STORheap  *{
    for(i=0,counter=0; i< VMEMBLOCKS; i++) {
        if ( Blocks[i].flag.used == 0 )   {                  {*  block |Qϥ  *{
             heapID->blocks[counter+heapID->actblocks]= i;   {* tm STORhrap      *{
             counter ++ ;                      {* ֭pwtm block Ӽ *{
             Blocks[i].flag.ID = heapID->ID ;  {* OѧOX              *{
             Blocks[i].flag.used = 1 ;         {* ܨϥΪA            *{
        }

        if (counter == incblocks)      {* Ŷ *{
            break;
    }


    {*  STORheap jp *{
    heapID->actblocks += incblocks ;


    {*  memory pool jp *{
    blksUsed  +=  incblocks ;



    #if (STORDEBUG)   {* q, CLXUƤe *{
    printf("aloSheap().   size = %lu  incblocks = %u \n", size, incblocks );
    Blocks_status_display();
    heapID_status_display(*heapID);
    #endif


    return Okay;
}
{*----------------------------------------------------------------------------*{
{* \ G   STORheap Ŷjp                                          *{
{*                                                                            *{
{* Ѽ G  struct STORheap  heapID : STORheap Ƶc                     *{
{*          unsigned long    *size  : ѼƱNǦ^ STORheap Ŷjp(bytes) *{
{*                                                                            *{
{* j G    POWRSTOR A                                                 *{
{*                                                                            *{
{* ~` G  W                    ӷ                                  *{
{*              L                                                            *{
{*                                                                            *{
{* `N G  ѩ POWRSTOR OH block AҥH *size NO                *{
{*          16384 (16K) ƭ                                              *{
{*----------------------------------------------------------------------------*{
STORstatus  sizSheap( struct STORheap  heapID,
                      unsigned long    *size
                    )
{
unsigned i ;



    if( pwrInitialized == false )  {* POWRSTOR |w *{
        return NotInitialized;


    *size = heapID.actblocks;  {* STORheap ujp (blocks) *{

    (*size) <<= 14;            {* ഫ blocks  bytes       *{

    return Okay;
}
{*----------------------------------------------------------------------------*{
{* \ G h STORheap ƨOwİ (DOS heap)                  *{
{*                                                                            *{
{* Ѽ G struct STORheap  heapID    : STORheap Ƶc                   *{
{*         unsigned long    fromSTOR  : STORhrap Ʒh_lm         *{
{*         void             *toMemory : OwİϪ_lm                *{
{*         unsigned         num_bytes : hƶq (bytes)                  *{
{*                                                                            *{
{* j G    POWRSTOR A                                                 *{
{*                                                                            *{
{* ~` G  W                    ӷ                                  *{
{*              printf()                C  Run Time {w                    *{
{*              heapID_status_display() POWRSTOR    {w                    *{
{*              EXPtoMem()              POWRDEVC    {w                    *{
{*              EXTget()                POWRDEVC    {w                    *{
{*              filGet()                POWRDEVC    {w                    *{
{*                                                                            *{
{* `N G b STORheap ƤevT                                   *{
{*                                                                            *{
{*         bOwİϪƤeNDл\                                 *{
{*----------------------------------------------------------------------------*{
STORstatus  getSTOR( struct STORheap  heapID,
                     unsigned long    fromSTOR,
                     void             *toMemory,
                     unsigned         num_bytes
                   )
{
unsigned blk;
unsigned blkAdr;
unsigned index;



    {* p [blk:blkAdr] w}覡 *{
    blk =  fromSTOR >> 14;                 {* ĴX block          *{
    blkAdr = fromSTOR - (blk << 14);       {* block ĴX byte *{



    if (blk > heapID.actblocks)   {* XzG *{
        return InvalidADR ;


    #if (STORDEBUG)  {* q, CLXUƤe *{
    printf("getSTOR() \n");
    printf("toMemory = %p    fromSTOR = %lu   num_bytes = %u   blk = %u  blkAdr = %u \n", \
            toMemory, fromSTOR, num_bytes, blk, blkAdr );
    heapID_status_display(heapID);
    #endif



    index = heapID.blocks[blk] ;
    switch ( Blocks[index].type ) {        {* @ذO髬A ? *{
        case  EXP_TYPE :  {* XiO *{
              blk = index - ExpFlag ;
              {* @ڪOh *{
              EXPtoMem( ExpFrame, blk, blkAdr, toMemory, ExpHandle, num_bytes );
              break;

        case  EXT_TYPE :  {* O *{
              blk = index - ExtFlag ;
              {* @ڪOh *{
              EXTget( ExtBase,  blk, blkAdr, toMemory, num_bytes ) ;
              break;

        case  DSK_TYPE :  {* ϺаO *{
              blk = index - FileFlag ;
              {* @ڪOh *{
              filGet( FileHandle, blk, blkAdr, toMemory, num_bytes );
              break;
    }


    #if (STORDEBUG)  {* q, CLXUƤe *{
    printf("blk = %u \n", blk);
    #endif

    return Okay;
}
{*----------------------------------------------------------------------------*{
{* \ G hOwİ (DOS heap) ƨ STORheap                  *{
{*                                                                            *{
{* Ѽ G struct STORheap  heapID      :  STORheap Ƶc                *{
{*         void             *fromMemory :  OwİϪ_l}             *{
{*         unsigned long    toSTOR      :  STORheap ƪ_lm        *{
{*         unsigned         num_bytes   :  h bytes Ӽ                  *{
{*                                                                            *{
{* j G    POWRSTOR A                                                 *{
{*                                                                            *{
{* ~` G  W                    ӷ                                  *{
{*              printf()                C  Run Time {w                    *{
{*              heapID_status_display() POWRSTOR    {w                    *{
{*              MemtoEXP()              POWRDEVC    {w                    *{
{*              EXTput()                POWRDEVC    {w                    *{
{*              filPut()                POWRDEVC    {w                    *{
{*                                                                            *{
{* `N G bOwİϤƤevT                               *{
{*                                                                            *{
{*         b STORheap ƤeNDл\                                 *{
{*----------------------------------------------------------------------------*{
STORstatus  putSTOR( struct STORheap  heapID,
                     void             *fromMemory,
                     unsigned long    toSTOR,
                     unsigned         num_bytes
                   )
{
unsigned blk;
unsigned blkAdr;
unsigned index;



    {* p [blk:blkAdr] w}覡 *{
    blk =  toSTOR >> 14;                   {* ĴX block          *{
    blkAdr = toSTOR - (blk << 14);         {* block ĴX byte *{



    if (blk > heapID.actblocks)            {* XzG *{
        return InvalidADR ;


    #if (STORDEBUG)  {* q, CLXUƤe *{
    printf("putSTOR() \n");
    printf("fromMemory = %p    toSTOR = %lu   num_bytes = %u   blk = %u  blkAdr = %u \n", \
            fromMemory, toSTOR, num_bytes, blk, blkAdr );
    heapID_status_display(heapID);
    #endif



    index = heapID.blocks[blk] ;
    switch ( Blocks[index].type ) {          {* @ذO髬A ? *{
        case  EXP_TYPE :    {* XiO *{
              blk = index - ExpFlag ;
              {* @ڪOh *{
              MemtoEXP( ExpFrame,  fromMemory, blk, blkAdr, ExpHandle, num_bytes );
              break;

        case  EXT_TYPE :    {* O *{
              blk = index - ExtFlag ;
              {* @ڪOh *{
              EXTput( ExtBase, fromMemory, blk, blkAdr, num_bytes ) ;
              break;

        case  DSK_TYPE :    {* ϺаO *{
              blk = index - FileFlag ;
              {* @ڪOh *{
              filPut( FileHandle, fromMemory, blk, blkAdr, num_bytes );
              break;
    }


    #if (STORDEBUG)  {* q, CLXUƤe *{
    printf("blk = %u \n", blk);
    #endif

    return Okay;
}
{*----------------------------------------------------------------------------*{
{* \ G ˬd POWRSTOR ANXC                                         *{
{*                                                                            *{
{*         pGO OkayAhjA                                              *{
{*         _h_{AXANXܿùWC                               *{
{*                                                                            *{
{* Ѽ G STORstatus  stat  :  POWRSTOR ANX                           *{
{*                                                                            *{
{* j G    POWRSTOR A                                                 *{
{*                                                                            *{
{* ~` G  W                    ӷ                                  *{
{*              abnormal()              POWRDEVC     {w                   *{
{*----------------------------------------------------------------------------*{
STORstatus  Sstatchk( STORstatus  stat )
{
    char  *errMsg;

    if( stat != Okay )
    {
        switch( stat )
        {
          case InvalInit      :  errMsg = "POWRSTOR MESSAGE: Invalid initialization??";
                                 break;
          case BadInitData    :  errMsg = "POWRSTOR MESSAGE: Bad initialization parameters??";
                                 break;
          case NotInitialized :  errMsg = "POWRSTOR MESSAGE: Not Active??";
                                 break;
          case NotEnoughMem   :  errMsg = "POWRSTOR MESSAGE: Not enough memory??";
                                 break;
          case FileERROR      :  errMsg = "POWRSTOR MESSAGE: Hard disk error??";
                                 break;
          case EXPerror       :  errMsg = "POWRSTOR MESSAGE: Expanded Memory error??";
                                 break;
          case EXTerror       :  errMsg = "POWRSTOR MESSAGE: Extended Memory error??";
                                 break;
          case InvalidADR     :  errMsg = "POWRSTOR MESSAGE: Invalid address argument??";
                                 break;
          case NoMoreMem      :  errMsg = "POWRSTOR MESSAGE: No more memory available??";
                                 break;
          case NoSuchHeap     :  errMsg = "POWRSTOR MESSAGE: No such STORheap??";
                                 break;
          case NoMoreHeaps    :  errMsg = "POWRSTOR MESSAGE: No more STORheaps available??";
                                 break;
          default             :  errMsg = "POWRSTOR MESSAGE: Unknown Status??";
                                 break;
        }
        abnormal( errMsg );   {* XTùWAõ{ *{
    }
    return stat;
}


#if (STORDEBUG)   {* q, CLXUƤe *{
{*----------------------------------------------------------------------------*{
{* \ G    uPOWRSTOR ޲z}Cve                                 *{
{*                                                                            *{
{* Ѽ G   L                                                               *{
{*                                                                            *{
{* j G    L                                                            *{
{*                                                                            *{
{* ~` G  W                    ӷ                                  *{
{*              printf()                C  Run Time {w                    *{
{*              getch()                 C  Run Time {w                    *{
{*----------------------------------------------------------------------------*{
static void Blocks_status_display(void)
{
int i;

    {* ܡuPOWRSTOR ޲z}CvC@Ӥ (N@ block)  *{
    {* O髬AAMϥΪA  (C 15 ƴN)          *{

    for( i=0; i<VMEMBLOCKS; i++)  {
         printf(" %2x%2x", Blocks[i].type,Blocks[i].flag);
         if (((i+1) % 15) == 0)   printf("\n");
    }
    printf("\n");
    getch();
}
{*----------------------------------------------------------------------------*{
{* \ G    STORheap T                                         *{
{*                                                                            *{
{* Ѽ G   struct STORheap heapID : STORheap Ƶc                     *{
{*                                                                            *{
{* j G    L                                                            *{
{*                                                                            *{
{* ~` G  W                    ӷ                                  *{
{*              printf()                C  Run Time {w                    *{
{*              getch()                 C  Run Time {w                    *{
{*----------------------------------------------------------------------------*{
static void heapID_status_display(struct STORheap heapID)
{
int i;

    {*  STORheap Ŷjp (blocks) *{
    printf(" heapID.actblocks = %u \n", heapID.actblocks);

    {*  STORheap ѧOX *{
    printf(" heapID.ID        = %u \n", heapID.ID);

    {*  STORheap C@ block O髬A *{
    {* C 15 ƴ@                        *{
    for (i=0 ; i< heapID.actblocks;  i++)  {
         printf(" %4u", heapID.blocks[i]);
         if (((i+1) % 15) == 0)   printf("\n");
    }

    printf("\n");
    getch();

}
{*----------------------------------------------------------------------------*{
#endif

{* ------------------------- end of file ---------------------------------- *{


